@@ -44,6 +44,7 @@
 #endif
 
 #include <glib/gi18n-lib.h>
+#include <string.h>
 
 static GogViewClass *gab_view_parent_klass;
 static GObjectClass *gab_parent_klass;
@@ -248,9 +249,8 @@ gog_axis_base_get_crossed_axis_type (GogAxisBase *axis_base)
 	axis_set = gog_chart_get_axis_set (axis_base->chart);
 
 	crossed_type = GOG_AXIS_UNKNOWN;
-	switch (axis_set) {
+	switch (axis_set & GOG_AXIS_SET_FUNDAMENTAL) {
 		case GOG_AXIS_SET_XY:
-		case GOG_AXIS_SET_XY_pseudo_3d:
 			if (axis_type == GOG_AXIS_X)
 				crossed_type = GOG_AXIS_Y;
 			else
@@ -268,6 +268,7 @@ gog_axis_base_get_crossed_axis_type (GogAxisBase *axis_base)
 		case GOG_AXIS_SET_XYZ:
 		case GOG_AXIS_SET_ALL:
 		case GOG_AXIS_SET_NONE:
+		default:
 			g_message ("[GogAxisBase::get_crossed_axis_type] unimplemented for this axis set (%i)",
 				   axis_set);
 			break;
@@ -376,6 +377,7 @@ cb_cross_axis_changed (GtkComboBox *combo, AxisBasePrefs *state)
 	GValue value;
 	GtkTreeModel *model = gtk_combo_box_get_model (combo);
 
+	memset (&value, 0, sizeof (GValue));
 	gtk_combo_box_get_active_iter (combo, &iter);
 	gtk_tree_model_get_value (model, &iter, 1, &value);
 	state->axis_base->crossed_axis_id = g_value_get_uint (&value);
@@ -383,6 +385,8 @@ cb_cross_axis_changed (GtkComboBox *combo, AxisBasePrefs *state)
 	gtk_toggle_button_set_active
 		(GTK_TOGGLE_BUTTON (glade_xml_get_widget (state->gui, "axis_cross")),
 		 TRUE);
+
+	g_value_unset (&value);
 }
 
 static void
@@ -625,48 +629,68 @@ gog_axis_base_class_init (GObjectClass *gobject_klass)
 	gog_klass->parent_changed 	= gog_axis_base_parent_changed;
 
 	g_object_class_install_property (gobject_klass, AXIS_BASE_PROP_POSITION,
-		g_param_spec_string ("pos_str", "pos_str",
-			"Where to position an axis low, high, or crossing",
-			"low", G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+		g_param_spec_string ("pos-str", 
+			_("Axis position"),
+			_("Where to position an axis low, high, or crossing"),
+			"low", 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, AXIS_BASE_PROP_MAJOR_TICK_LABELED,
-		g_param_spec_boolean ("major-tick-labeled", NULL,
-			"Show labels for major ticks",
-			TRUE, G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+		g_param_spec_boolean ("major-tick-labeled", 
+			_("Major labels"),
+			_("Show labels for major ticks"),
+			TRUE, 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, AXIS_BASE_PROP_MAJOR_TICK_IN,
-		g_param_spec_boolean ("major-tick-in", NULL,
-			"Major tick marks inside the axis",
-			FALSE, G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+		g_param_spec_boolean ("major-tick-in", 
+			_("Inside major ticks"),
+			_("Major tick marks inside the chart area"),
+			FALSE, 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, AXIS_BASE_PROP_MAJOR_TICK_OUT,
-		g_param_spec_boolean ("major-tick-out", NULL,
-			"Major tick marks outside the axis",
-			TRUE, G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+		g_param_spec_boolean ("major-tick-out", 
+			_("Outside major ticks"),
+			_("Major tick marks outside the chart area"),
+			TRUE, 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, AXIS_BASE_PROP_MAJOR_TICK_SIZE_PTS,
-		g_param_spec_int ("major-tick-size-pts", "major-tick-size-pts",
-			"Size of the major tick marks, in points",
-			0, 20, 4, G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+		g_param_spec_int ("major-tick-size-pts", 
+			_("Major tick size"),
+			_("Size of the major tick marks, in points"),
+			0, 20, 4, 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 
 	g_object_class_install_property (gobject_klass, AXIS_BASE_PROP_MINOR_TICK_IN,
-		g_param_spec_boolean ("minor-tick-in", NULL,
-			"Minor tick marks inside the axis",
-			FALSE, G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+		g_param_spec_boolean ("minor-tick-in", 
+			_("Inside minor ticks"),
+			_("Minor tick marks inside the chart area"),
+			FALSE, 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, AXIS_BASE_PROP_MINOR_TICK_OUT,
-		g_param_spec_boolean ("minor-tick-out", NULL,
-			"Minor tick marks outside the axis",
-			FALSE, G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+		g_param_spec_boolean ("minor-tick-out", 
+			_("Outside minor ticks"),
+			_("Minor tick marks outside the axis"),
+			FALSE, 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, AXIS_BASE_PROP_MINOR_TICK_SIZE_PTS,
-		g_param_spec_int ("minor-tick-size-pts", NULL,
-			"Size of the minor tick marks, in points",
-			0, 15, 2, G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+		g_param_spec_int ("minor-tick-size-pts", 
+			_("Minor tick size"),
+			_("Size of the minor tick marks, in points"),
+			0, 15, 2, 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 
 	g_object_class_install_property (gobject_klass, AXIS_BASE_PROP_CROSS_AXIS_ID,
-		g_param_spec_uint ("cross_axis_id", NULL,
-			"Which axis to cross",
-			0, G_MAXUINT, 0, G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+		g_param_spec_uint ("cross-axis-id", 
+			_("Cross axis id"),
+			_("Which axis to cross"),
+			0, G_MAXUINT, 0, 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 
 	g_object_class_install_property (gobject_klass, AXIS_BASE_PROP_PADDING_PTS,
-		g_param_spec_int ("padding-pts", NULL,
-			"Distance from axis line to plot area, in points",
-			-G_MAXINT, G_MAXINT, 0, G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+		g_param_spec_int ("padding-pts", 
+			_("Axis padding"),
+			_("Distance from axis line to plot area, in points"),
+			-G_MAXINT, G_MAXINT, 0, 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 
 #ifdef GOFFICE_WITH_GTK
 	gog_klass->populate_editor	= gog_axis_base_populate_editor;
@@ -925,12 +949,15 @@ axis_line_get_bbox (GogAxisBase *axis_base, GogRenderer *renderer,
 	GogStyle *style = axis_base->base.style;
 	GOGeometryAABR txt_aabr;
 	GOGeometryOBR txt_obr;
+	GOGeometryOBR *obrs = NULL;
+	GOGeometrySide label_anchor = GO_SIDE_AUTO;
 	double line_width;
 	double axis_length, axis_angle, label_padding;
 	double cos_alpha, sin_alpha;
 	double pos;
 	double minor_tick_len, major_tick_len, tick_len;
 	double padding = gog_axis_base_get_padding (axis_base);
+	double label_size_max = 0;
 	unsigned i, tick_nbr;
 	gboolean is_line_visible;
 
@@ -945,7 +972,7 @@ axis_line_get_bbox (GogAxisBase *axis_base, GogRenderer *renderer,
 	major_tick_len = gog_renderer_pt2r (renderer, axis_base->major.size_pts);
 	tick_len = axis_base->major.tick_out ? major_tick_len :
 		(axis_base->minor.tick_out ? minor_tick_len : 0.);
-	gog_renderer_get_text_OBR (renderer, "0", &txt_obr);
+	gog_renderer_get_text_OBR (renderer, "0", TRUE, &txt_obr);
 	label_padding = txt_obr.w;
 
 	total_bbox.x = x; total_bbox.y = y; 
@@ -982,23 +1009,39 @@ axis_line_get_bbox (GogAxisBase *axis_base, GogRenderer *renderer,
 
 	tick_nbr = gog_axis_get_ticks (axis_base->axis, &ticks);
 
-	if (!draw_labels)
+	if (!draw_labels || tick_nbr < 1)
 		return total_bbox;
 
 	map = gog_axis_map_new (axis_base->axis, 0., axis_length);
 
+	obrs = g_new0 (GOGeometryOBR, tick_nbr);
 	for (i = 0; i < tick_nbr; i++) {
 		if (ticks[i].label != NULL) {
+			GOGeometryOBR *obr = obrs + i;
+			gog_renderer_get_text_OBR (renderer, ticks[i].label, TRUE, obr);
+			if (obr->w > label_size_max
+			    || obr->h > label_size_max) {
+				label_size_max = MAX (obr->w, obr->h);
+				label_anchor = go_geometry_calc_label_anchor (obr, axis_angle);
+			}					
+		}
+	}
+
+	for (i = 0; i < tick_nbr; i++) {
+		if (ticks[i].label != NULL) {
+			GOGeometryOBR *obr = obrs + i;
 			pos = gog_axis_map_to_view (map, ticks[i].position);
-			gog_renderer_get_text_OBR (renderer, ticks[i].label, &txt_obr);
-			txt_obr.w += label_padding;
-			go_geometry_calc_label_position (&txt_obr, axis_angle, tick_len, side);
-			txt_obr.x += x + pos * cos (axis_angle);
-			txt_obr.y += y + pos * sin (axis_angle);
-			go_geometry_OBR_to_AABR (&txt_obr, &txt_aabr);
+			obr->w += label_padding;
+			go_geometry_calc_label_position (obr, axis_angle, tick_len, 
+							 side, label_anchor);
+			obr->x += x + pos * cos (axis_angle);
+			obr->y += y + pos * sin (axis_angle);
+			go_geometry_OBR_to_AABR (obr, &txt_aabr);
 			go_geometry_AABR_add (&total_bbox, &txt_aabr);
 		}
 	}
+	g_free (obrs);
+
 	gog_axis_map_free (map);
 
 	return total_bbox;
@@ -1016,9 +1059,10 @@ axis_line_render (GogAxisBase *axis_base,
 {
 	GogAxisMap *map = NULL;
 	GogAxisTick *ticks;
-	GogViewAllocation label_pos;
 	GogStyle *style = axis_base->base.style;
-	GOGeometryOBR txt_obr, txt_obr_old = {0., 0., 0., 0., 0.};
+	GOGeometryOBR zero_obr;
+	GOGeometryOBR *obrs = NULL;
+	GOGeometrySide label_anchor = GO_SIDE_AUTO;
 	ArtVpath path[3];
 	double line_width;
 	double axis_length, axis_angle, label_padding;
@@ -1028,7 +1072,8 @@ axis_line_render (GogAxisBase *axis_base,
 	double cos_alpha, sin_alpha;
 	double pos, pos_x, pos_y;
 	double padding = gog_axis_base_get_padding (axis_base);
-	unsigned i, tick_nbr;
+	double label_size_max = 0;
+	unsigned i, tick_nbr, nobr = 0, *indexmap = NULL;
 	gboolean draw_major, draw_minor;
 	gboolean is_line_visible;
 
@@ -1081,10 +1126,25 @@ axis_line_render (GogAxisBase *axis_base,
 
 	tick_len = axis_base->major.tick_out ? major_tick_len :
 		(axis_base->minor.tick_out ? minor_tick_len : 0.);
-	gog_renderer_get_text_OBR (renderer, "0", &txt_obr);
-	label_padding = txt_obr.w;
+	gog_renderer_get_text_OBR (renderer, "0", TRUE, &zero_obr);
+	label_padding = zero_obr.w;
 
 	tick_nbr = gog_axis_get_ticks (axis_base->axis, &ticks);
+	if (draw_labels) {
+		obrs = g_new0 (GOGeometryOBR, tick_nbr);
+		indexmap = g_new0 (unsigned int, tick_nbr);
+		for (i = 0; i < tick_nbr; i++) {
+			if (ticks[i].label != NULL) {
+				GOGeometryOBR *obr = obrs + i;
+				gog_renderer_get_text_OBR (renderer, ticks[i].label, TRUE, obr);
+				if (obr->w > label_size_max
+				    || obr->h > label_size_max) {
+					label_size_max = MAX (obr->w, obr->h);
+					label_anchor = go_geometry_calc_label_anchor (obr, axis_angle);
+				}					
+			}
+		}
+	}
 
 	for (i = 0; i < tick_nbr; i++) {
 		if (gog_axis_map (map, ticks[i].position) < start_at)
@@ -1128,22 +1188,54 @@ axis_line_render (GogAxisBase *axis_base,
 		}
 
 		if (ticks[i].label != NULL && draw_labels) {
+			GOGeometryOBR *obr = obrs + i;
 			pos = gog_axis_map_to_view (map, ticks[i].position);
-			gog_renderer_get_text_OBR (renderer, ticks[i].label, &txt_obr);
-			txt_obr.w += label_padding;
-			go_geometry_calc_label_position (&txt_obr, axis_angle, tick_len, side);
-			txt_obr.x += x + pos * cos (axis_angle);
-			txt_obr.y += y + pos * sin (axis_angle);
-			if (!go_geometry_test_OBR_overlap (&txt_obr, &txt_obr_old)) {
-				label_pos.x = txt_obr.x;
-				label_pos.y = txt_obr.y;
-				gog_renderer_draw_text (renderer, ticks[i].label,
-							&label_pos, GTK_ANCHOR_CENTER, NULL);
-				txt_obr_old = txt_obr;
-			}
+			obr->w += label_padding;
+			go_geometry_calc_label_position (obr, axis_angle, tick_len, 
+							 side, label_anchor);
+			obr->x += x + pos * cos (axis_angle);
+			obr->y += y + pos * sin (axis_angle);
+
+			indexmap[nobr] = i;
+			nobr++;
 		}
 	}
 
+	/*
+	 * This is far from perfect, but at least things are regular.
+	 * The axis really needs to be queried to figure this out,
+	 * especially if dates are involved.
+	 */
+	if (nobr > 0) {
+		int skip = 1;
+
+		for (i = skip; i < nobr; i += skip) {
+			unsigned j;
+			gboolean overlap;
+
+			j = indexmap[i];
+			overlap = go_geometry_test_OBR_overlap (obrs + j,
+								obrs + indexmap[i - skip]);
+			if (overlap) {
+				skip++;
+				i = 0;
+				continue;
+			}
+		}
+
+		for (i = 0; i < nobr; i += skip) {
+			unsigned j = indexmap[i];
+			GogViewAllocation label_pos;
+			label_pos.x = obrs[j].x;
+			label_pos.y = obrs[j].y;
+			gog_renderer_draw_text (renderer, ticks[j].label,
+						&label_pos, GTK_ANCHOR_CENTER,
+						TRUE);
+		}
+	}
+	g_free (obrs);
+	g_free (indexmap);
+
 	gog_axis_map_free (map);
 }
 
@@ -1193,7 +1285,7 @@ axis_circle_get_bbox (GogAxisBase *axis_base, GogRenderer *renderer,
 	major_tick_len = gog_renderer_pt2r (renderer, axis_base->major.size_pts);
 	tick_len = axis_base->major.tick_out ? major_tick_len :
 		(axis_base->minor.tick_out ? minor_tick_len : 0.);
-	gog_renderer_get_text_OBR (renderer, "0", &txt_obr);
+	gog_renderer_get_text_OBR (renderer, "0", TRUE, &txt_obr);
 	label_padding = txt_obr.w;
 
 	draw_ticks = gog_style_is_line_visible (axis_base->base.style) &&
@@ -1208,9 +1300,10 @@ axis_circle_get_bbox (GogAxisBase *axis_base, GogRenderer *renderer,
 		gog_chart_map_2D_to_view (c_map, ticks[i].position, position, &x, &y);
 
 		if (ticks[i].label != NULL && draw_labels) {
-			gog_renderer_get_text_OBR (renderer, ticks[i].label, &txt_obr);
+			gog_renderer_get_text_OBR (renderer, ticks[i].label, TRUE, &txt_obr);
 			txt_obr.w += label_padding;
-			go_geometry_calc_label_position (&txt_obr, angle + M_PI / 2.0, tick_len, GO_SIDE_LEFT);
+			go_geometry_calc_label_position (&txt_obr, angle + M_PI / 2.0, tick_len, 
+							 GO_SIDE_LEFT, GO_SIDE_AUTO);
 			txt_obr.x += x;
 			txt_obr.y += y;
 			go_geometry_OBR_to_AABR (&txt_obr, &txt_aabr);
@@ -1282,7 +1375,7 @@ axis_circle_render (GogAxisBase *axis_base, GogRenderer *renderer,
 	major_tick_len = gog_renderer_pt2r (renderer, axis_base->major.size_pts);
 	tick_len = axis_base->major.tick_out ? major_tick_len :
 		(axis_base->minor.tick_out ? minor_tick_len : 0.);
-	gog_renderer_get_text_OBR (renderer, "0", &txt_obr);
+	gog_renderer_get_text_OBR (renderer, "0", TRUE, &txt_obr);
 	label_padding = txt_obr.w;
 
 	tick_nbr = gog_axis_get_ticks (axis_base->axis, &ticks);
@@ -1336,9 +1429,10 @@ axis_circle_render (GogAxisBase *axis_base, GogRenderer *renderer,
 		if (ticks[i].label != NULL && draw_labels) {
 			gog_chart_map_2D_to_view (c_map, ticks[i].position, position,
 						  &label_pos.x, &label_pos.y);
-			gog_renderer_get_text_OBR (renderer, ticks[i].label, &txt_obr);
+			gog_renderer_get_text_OBR (renderer, ticks[i].label, TRUE, &txt_obr);
 			txt_obr.w += label_padding;
-			go_geometry_calc_label_position (&txt_obr, angle + M_PI / 2.0, tick_len, GO_SIDE_LEFT);
+			go_geometry_calc_label_position (&txt_obr, angle + M_PI / 2.0, tick_len, 
+							 GO_SIDE_LEFT, GO_SIDE_AUTO);
 			label_pos.x += txt_obr.x;
 			label_pos.y += txt_obr.y;
 			txt_obr.x = label_pos.x;
@@ -1347,7 +1441,7 @@ axis_circle_render (GogAxisBase *axis_base, GogRenderer *renderer,
 			    (!go_geometry_test_OBR_overlap (&txt_obr, &txt_obr_old) &&
 			     !go_geometry_test_OBR_overlap (&txt_obr, &txt_obr_first))) {
 				gog_renderer_draw_text (renderer, ticks[i].label,
-							&label_pos, GTK_ANCHOR_CENTER, NULL);
+							&label_pos, GTK_ANCHOR_CENTER, TRUE);
 				txt_obr_old = txt_obr;
 			}
 			if (!first_label_done) {
@@ -1669,21 +1763,19 @@ gog_axis_base_view_padding_request (GogView *view, GogViewAllocation const *bbox
 	GogAxisBase *axis_base = GOG_AXIS_BASE (view->model);
 	GogStyle *style = axis_base->base.style;
 
+	if (gog_axis_get_atype (axis_base->axis) >= GOG_AXIS_VIRTUAL)
+		return;
 	axis_set = gog_chart_get_axis_set (axis_base->chart);
 
 	gog_renderer_push_style (view->renderer, style);
 
-	switch (axis_set) {
+	switch (axis_set & GOG_AXIS_SET_FUNDAMENTAL) {
 		case GOG_AXIS_SET_X:
 			x_process (GOG_AXIS_BASE_PADDING_REQUEST, view, padding, bbox, 0., 0.);
 			break;
 		case GOG_AXIS_SET_XY:
 			xy_process (GOG_AXIS_BASE_PADDING_REQUEST, view, padding, bbox, 0., 0.);
 			break;
-		case GOG_AXIS_SET_XY_pseudo_3d:
-			if (gog_axis_get_atype (axis_base->axis) != GOG_AXIS_PSEUDO_3D)
-				xy_process (GOG_AXIS_BASE_PADDING_REQUEST, view, padding, bbox, 0., 0.);
-			break;
 		case GOG_AXIS_SET_RADAR:
 			radar_process (GOG_AXIS_BASE_PADDING_REQUEST, view, padding, bbox, 0., 0.);
 			break;
@@ -1706,6 +1798,9 @@ gog_axis_base_view_render (GogView *view, GogViewAllocation const *bbox)
 	GogStyle *style = axis_base->base.style;
 	GogViewAllocation const *plot_area;
 
+	if (gog_axis_get_atype (axis_base->axis) >= GOG_AXIS_VIRTUAL)
+		return;
+
 	axis_set = gog_chart_get_axis_set (axis_base->chart);
 	/* FIXME: not nice */
 	if (IS_GOG_AXIS (view->model))
@@ -1715,17 +1810,13 @@ gog_axis_base_view_render (GogView *view, GogViewAllocation const *bbox)
 
 	gog_renderer_push_style (view->renderer, style);
 
-	switch (axis_set) {
+	switch (axis_set & GOG_AXIS_SET_FUNDAMENTAL) {
 		case GOG_AXIS_SET_X:
 			x_process (GOG_AXIS_BASE_RENDER, view, NULL, plot_area, 0., 0.);
 			break;
 		case GOG_AXIS_SET_XY:
 			xy_process (GOG_AXIS_BASE_RENDER, view, NULL, plot_area, 0., 0.);
 			break;
-		case GOG_AXIS_SET_XY_pseudo_3d:
-			if (gog_axis_get_atype (axis_base->axis) != GOG_AXIS_PSEUDO_3D)
-				xy_process (GOG_AXIS_BASE_RENDER, view, NULL, plot_area, 0., 0.);
-			break;
 		case GOG_AXIS_SET_RADAR:
 			radar_process (GOG_AXIS_BASE_RENDER, view, NULL, plot_area, 0., 0.);
 			break;
