@@ -53,7 +53,8 @@ static const struct {
 	{ "xy", 	GOG_AXIS_SET_XY},
 	{ "xyz",	GOG_AXIS_SET_XYZ},
 	{ "radar",	GOG_AXIS_SET_RADAR},
-	{ "pseudo-3d",	GOG_AXIS_SET_XY_pseudo_3d}
+	{ "pseudo-3d",	GOG_AXIS_SET_XY_pseudo_3d},
+	{ "xy-color",	GOG_AXIS_SET_XY_COLOR}
 };
 	
 GogAxisSet
@@ -125,8 +126,8 @@ calc_circle_parameters (GogViewAllocation const *area, GogChartMapPolarData *dat
 			data->th1 -= x;
 		} else if (data->th1 > 2.0 * M_PI) {
 			x = data->th1 - fmod (data->th1, 2.0 * M_PI);
-			data->th0 += x;
-			data->th1 += x;
+			data->th0 -= x;
+			data->th1 -= x;
 		}
 		if (data->th1 - data->th0 > go_add_epsilon (2 * M_PI)) 
 			data->th1 = data->th0 + 2 * M_PI; 
@@ -254,7 +255,7 @@ gog_chart_map_new (GogChart *chart, GogViewAllocation const *area,
 	map->is_valid = FALSE;
 
 	axis_set = gog_chart_get_axis_set (chart);
-	switch (axis_set) {
+	switch (axis_set & GOG_AXIS_SET_FUNDAMENTAL) {
 		case GOG_AXIS_SET_X:
 			{
 				XMapData *data = g_new (XMapData, 1);
@@ -273,6 +274,7 @@ gog_chart_map_new (GogChart *chart, GogViewAllocation const *area,
 			}
 		case GOG_AXIS_SET_XY:
 		case GOG_AXIS_SET_XY_pseudo_3d:
+		case GOG_AXIS_SET_XY_COLOR:
 			{
 				map->axis_map[0] = gog_axis_map_new (axis0, map->area.x, map->area.w);
 				map->axis_map[1] = gog_axis_map_new (axis1, map->area.y + map->area.h, 
@@ -328,6 +330,7 @@ gog_chart_map_new (GogChart *chart, GogViewAllocation const *area,
 		case GOG_AXIS_SET_ALL:
 		case GOG_AXIS_SET_NONE:
 		case GOG_AXIS_SET_UNKNOWN:
+		default:
 			g_warning ("[Chart::map_new] not implemented for this axis set (%i)",
 				   axis_set);
 			map->map_2D_to_view = null_map_2D;
@@ -550,7 +553,30 @@ role_plot_post_add (GogObject *parent, GogObject *plot)
 {
 	GogChart *chart = GOG_CHART (parent);
 	gboolean ok = TRUE;
+	GogPlotClass *plot_klass = GOG_PLOT_CLASS (G_OBJECT_GET_CLASS (plot));
+	GogAxisSet axis_set = plot_klass->axis_set & ~GOG_AXIS_SET_FUNDAMENTAL;
 
+	if (axis_set) {
+		int i = GOG_AXIS_VIRTUAL, j = 1 << GOG_AXIS_VIRTUAL;
+		for (; i < GOG_AXIS_TYPES; i++, j <<= 1)
+			if ((axis_set & j) != 0 && (chart->axis_set & j) == 0) {
+				GogObject *axis = GOG_OBJECT (g_object_new (GOG_AXIS_TYPE, "type", i, NULL));
+				chart->axis_set |= j;
+				switch (i) {
+				case GOG_AXIS_PSEUDO_3D:
+					gog_object_add_by_name (GOG_OBJECT (chart), "Pseudo-3D-Axis", axis);
+					break;
+				case GOG_AXIS_COLOR:
+					gog_object_add_by_name (GOG_OBJECT (chart), "Color-Axis", axis);
+					break;
+				case GOG_AXIS_BUBBLE:
+					gog_object_add_by_name (GOG_OBJECT (chart), "Bubble-Axis", axis);
+					break;
+				default:
+					g_warning ("Unknown axis type: %x\n", i);
+				}
+			}
+	}
 	/* APPEND to keep order, there won't be that many */
 	chart->plots = g_slist_append (chart->plots, plot);
 	gog_chart_request_cardinality_update (chart);
@@ -580,6 +606,7 @@ role_plot_pre_remove (GogObject *parent, GogObject *plot)
 	    chart->axis_set != GOG_AXIS_SET_XY &&
 	    chart->axis_set != GOG_AXIS_SET_X && 
 	    chart->axis_set != GOG_AXIS_SET_XY_pseudo_3d &&
+	    chart->axis_set != GOG_AXIS_SET_XY_COLOR &&
 	    chart->axis_set != GOG_AXIS_SET_RADAR) {
 		GogObject *grid = chart->grid; /* clear_parent clears ::grid */
 		gog_object_clear_parent (GOG_OBJECT (grid));
@@ -595,6 +622,7 @@ role_grid_can_add (GogObject const *parent)
 		(chart->axis_set == GOG_AXIS_SET_XY ||
 		 chart->axis_set == GOG_AXIS_SET_X ||
 		 chart->axis_set == GOG_AXIS_SET_XY_pseudo_3d ||
+		 chart->axis_set == GOG_AXIS_SET_XY_COLOR ||
 		 chart->axis_set == GOG_AXIS_SET_RADAR);
 }
 
@@ -659,6 +687,10 @@ static gboolean radial_axis_can_add (GogObject const *parent) { return axis_can_
 static void radial_axis_post_add    (GogObject *parent, GogObject *child)  { axis_post_add   (child, GOG_AXIS_RADIAL); }
 static gboolean pseudo_3d_axis_can_add (GogObject const *parent) { return axis_can_add (parent, GOG_AXIS_PSEUDO_3D); }
 static void pseudo_3d_axis_post_add    (GogObject *parent, GogObject *child)  { axis_post_add   (child, GOG_AXIS_PSEUDO_3D); }
+static gboolean bubble_axis_can_add (GogObject const *parent) { return axis_can_add (parent, GOG_AXIS_BUBBLE); }
+static void bubble_axis_post_add    (GogObject *parent, GogObject *child)  { axis_post_add   (child, GOG_AXIS_BUBBLE); }
+static gboolean color_axis_can_add (GogObject const *parent) { return axis_can_add (parent, GOG_AXIS_COLOR); }
+static void color_axis_post_add    (GogObject *parent, GogObject *child)  { axis_post_add   (child, GOG_AXIS_COLOR); }
 
 static GogObjectRole const roles[] = {
 	{ N_("Grid"), "GogGrid",	0,
@@ -688,7 +720,15 @@ static GogObjectRole const roles[] = {
 	  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
 	  pseudo_3d_axis_can_add, axis_can_remove, NULL, pseudo_3d_axis_post_add, axis_pre_remove, NULL,
 	  { GOG_AXIS_PSEUDO_3D } },
-	{ N_("Plot"), "GogPlot",	4,	/* keep the axis before the plots */
+	{ N_("Bubble-Axis"), "GogAxis", 4,
+	  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
+	  bubble_axis_can_add, axis_can_remove, NULL, bubble_axis_post_add, axis_pre_remove, NULL,
+	  { GOG_AXIS_BUBBLE } },
+	{ N_("Color-Axis"), "GogAxis", 5,
+	  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_ROLE,
+	  color_axis_can_add, axis_can_remove, NULL, color_axis_post_add, axis_pre_remove, NULL,
+	  { GOG_AXIS_COLOR } },
+	{ N_("Plot"), "GogPlot",	6,	/* keep the axis before the plots */
 	  GOG_POSITION_SPECIAL, GOG_POSITION_SPECIAL, GOG_OBJECT_NAME_BY_TYPE,
 	  NULL, NULL, NULL, role_plot_post_add, role_plot_pre_remove, NULL, { -1 } },
 	{ N_("Title"), "GogLabel",	10,
@@ -718,17 +758,23 @@ gog_chart_class_init (GogObjectClass *gog_klass)
 	gog_klass->can_manual_size = TRUE;
 
 	g_object_class_install_property (gobject_klass, CHART_PROP_CARDINALITY_VALID,
-		g_param_spec_boolean ("cardinality-valid", "cardinality-valid",
-				      "Is the charts cardinality currently valid",
-				      FALSE, G_PARAM_READABLE));
+		g_param_spec_boolean ("cardinality-valid", 
+				      _("Valid cardinality"),
+				      _("Is the charts cardinality currently valid"),
+				      FALSE, 
+				      GSF_PARAM_STATIC | G_PARAM_READABLE));
 	g_object_class_install_property (gobject_klass, CHART_PROP_PLOT_AREA,
-		g_param_spec_string ("plot-area", "Plot area",
-				     "Position and size of plot area, in percentage of chart size",
-				     "0 0 1 1", G_PARAM_READWRITE|GOG_PARAM_PERSISTENT));
+		g_param_spec_string ("plot-area", 
+				     _("Plot area"),
+				     _("Position and size of plot area, in percentage of chart size"),
+				     "0 0 1 1", 
+				     GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, CHART_PROP_PLOT_AREA_IS_MANUAL,
-		g_param_spec_boolean ("is-plot-area-manual", "Is plot area manual", 
-				      "Is plot area manual",
-				      FALSE, G_PARAM_READWRITE|GOG_PARAM_PERSISTENT));
+		g_param_spec_boolean ("is-plot-area-manual", 
+				      _("Manual plot area"), 
+				      _("Is plot area manual"),
+				      FALSE, 
+				      GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 
 	gog_klass->view_type = gog_chart_view_get_type ();
 	gog_klass->update    = gog_chart_update;
@@ -832,7 +878,7 @@ gog_chart_get_plot_area (GogChart *chart, GogViewAllocation *plot_area)
 }
 
 /**
- * gog_chart_set_plot_area :
+ * gog_chart_set_plot_area:
  * @chart : #GogChart
  * @plot_area  : #GogViewAllocation
  *
@@ -852,7 +898,19 @@ gog_chart_set_plot_area (GogChart *chart, GogViewAllocation const *plot_area)
 	gog_object_emit_changed (GOG_OBJECT (chart), TRUE);
 }
 
-/* FIXME: function description here */
+/**
+ * gog_chart_get_cardinality:
+ * @chart: a #GogChart
+ * @full: placeholder for full cardinality
+ * @visible: placeholder for visible cardinality
+ *
+ * Update and cache cardinality values if required, and returns
+ * full and visible cardinality. Full cardinality is the number of 
+ * chart elements that require a different style. Visible cardinality is
+ * the number of chart elements shown in chart legend.
+ *
+ * @full and @visible may be NULL.
+ **/ 
 void
 gog_chart_get_cardinality (GogChart *chart, unsigned *full, unsigned *visible)
 {
@@ -865,6 +923,7 @@ gog_chart_get_cardinality (GogChart *chart, unsigned *full, unsigned *visible)
 		chart->cardinality_valid = TRUE;
 		chart->full_cardinality = chart->visible_cardinality = 0;
 		for (ptr = chart->plots ; ptr != NULL ; ptr = ptr->next) {
+			gog_plot_update_cardinality (ptr->data, chart->full_cardinality);
 			gog_plot_get_cardinality (ptr->data, &tmp_full, &tmp_visible);
 			chart->full_cardinality += tmp_full;
 			chart->visible_cardinality += tmp_visible;
