@@ -43,6 +43,14 @@
 
 /*****************************************************************************/
 
+/**
+ * gog_editor_new:
+ *
+ * Creates a new GogEditor object, which is used to store a 
+ * collection of property edition widgets (pages). The returned 
+ * object must be freed using @gog_editor_free.
+ **/
+
 GogEditor *
 gog_editor_new (void)
 {
@@ -50,10 +58,37 @@ gog_editor_new (void)
 
 	editor->store_page = NULL;
 	editor->pages = NULL;
+	g_datalist_init (&editor->registered_widgets);
 
 	return editor;
 }
 
+/**
+ * gog_editor_free:
+ * @editor: a #GogEditor
+ *
+ * Frees a GogEditor object.
+ **/
+
+void
+gog_editor_free (GogEditor *editor)
+{
+	g_slist_foreach (editor->pages, (GFunc) g_free, NULL);
+	g_slist_free (editor->pages);
+	g_datalist_clear (&editor->registered_widgets);
+
+	g_free (editor);
+}
+
+/**
+ * gog_editor_add_page:
+ * @editor: a #GogEditor
+ * @widget: property edition widget
+ * @label: a label identifying the widget
+ *
+ * Adds a page to @editor.
+ */
+
 void
 gog_editor_add_page (GogEditor *editor, gpointer widget, char const *label)
 {
@@ -68,6 +103,14 @@ gog_editor_add_page (GogEditor *editor, gpointer widget, char const *label)
 	editor->pages = g_slist_prepend (editor->pages, page);
 }
 
+/**
+ * gog_editor_set_store_page:
+ * @editor: a #GogEditor
+ * @store_page: placeholder for the last selected page
+ *
+ * Sets a placeholder for storing the last active editor page.
+ **/
+
 void
 gog_editor_set_store_page (GogEditor *editor, unsigned *store_page)
 {
@@ -77,6 +120,42 @@ gog_editor_set_store_page (GogEditor *editor, unsigned *store_page)
 }
 
 #ifdef GOFFICE_WITH_GTK
+
+/**
+ * gog_editor_register_widget:
+ * @editor: a #GogEditor
+ * @widget: a #GtkWidget
+ * 
+ * Registers a widget that then can be retrieved later using 
+ * @gog_editor_get_registered_widget. The main use of this function is to 
+ * provide the ability to extend a page.
+ **/
+
+void
+gog_editor_register_widget (GogEditor *editor, GtkWidget *widget)
+{
+	g_return_if_fail (editor != NULL);
+	g_return_if_fail (GTK_IS_WIDGET (widget));
+
+	g_datalist_set_data (&editor->registered_widgets, gtk_widget_get_name (widget), widget);
+}
+
+/**
+ * gog_editor_get_registered_widget:
+ * @editor: a #GogEditor
+ * @name: the name of the registered widget
+ *
+ * Retrieves a widget previously registered using @gog_editor_register_widget.
+ **/
+
+GtkWidget *
+gog_editor_get_registered_widget (GogEditor *editor, char const *name)
+{
+	g_return_val_if_fail (editor != NULL, NULL);
+
+	return g_datalist_get_data (&editor->registered_widgets, name);
+}
+
 static void
 cb_switch_page (G_GNUC_UNUSED GtkNotebook *n, G_GNUC_UNUSED GtkNotebookPage *p,
 		guint page_num, guint *store_page)
@@ -84,7 +163,14 @@ cb_switch_page (G_GNUC_UNUSED GtkNotebook *n, G_GNUC_UNUSED GtkNotebookPage *p,
 		*store_page = page_num;
 }
 
-gpointer
+/**
+ * gog_editor_get_notebook:
+ * @editor: a #GogEditor
+ *
+ * Buils a GtkNotebook from the widget collection stored in @editor.
+ **/
+
+GtkWidget *
 gog_editor_get_notebook (GogEditor *editor)
 {
 	GtkWidget *notebook;
@@ -121,17 +207,9 @@ gog_editor_get_notebook (GogEditor *editor)
 
 	return notebook;
 }
+
 #endif
 
-void
-gog_editor_free (GogEditor *editor)
-{
-	g_slist_foreach (editor->pages, (GFunc) g_free, NULL);
-	g_slist_free (editor->pages);
-
-	g_free (editor);
-}
-
 typedef struct {
 	char const *label;
 	char const *value;
@@ -655,29 +733,41 @@ gog_object_class_init (GObjectClass *klass)
 	gog_klass->use_parent_as_proxy = FALSE;
 
 	g_object_class_install_property (klass, OBJECT_PROP_ID,
-		g_param_spec_uint ("id", "id", "Object ID",
-				   0, G_MAXINT, 0,
-				   G_PARAM_READWRITE|GOG_PARAM_PERSISTENT));
+		g_param_spec_uint ("id", 
+			_("Object ID"), 
+			_("Object numerical ID"),
+			0, G_MAXINT, 0,
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	g_object_class_install_property (klass, OBJECT_PROP_POSITION,
-		g_param_spec_string ("position", "Position", 
-				     "Position and size of object, in percentage of parent size",
-				     "0 0 1 1", G_PARAM_READWRITE|GOG_PARAM_PERSISTENT));
+		g_param_spec_string ("position", 
+			_("Position"), 
+			_("Position and size of object, in percentage of parent size"),
+			"0 0 1 1", 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE|GOG_PARAM_PERSISTENT));
 	g_object_class_install_property (klass, OBJECT_PROP_POSITION_COMPASS,
-		g_param_spec_string ("compass", "Compass",
-				     "Compass auto position flags",
-				     "top", G_PARAM_READWRITE|GOG_PARAM_PERSISTENT));
+		g_param_spec_string ("compass", 
+			_("Compass"),
+			_("Compass auto position flags"),
+			"top", 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT | GOG_PARAM_POSITION));
 	g_object_class_install_property (klass, OBJECT_PROP_POSITION_ALIGNMENT,
-		g_param_spec_string ("alignment", "Alignment",
-				     "Alignment flag",
-				     "fill", G_PARAM_READWRITE|GOG_PARAM_PERSISTENT));
+		g_param_spec_string ("alignment", 
+			_("Alignment"),
+			_("Alignment flag"),
+			"fill",
+		       	GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT | GOG_PARAM_POSITION));
 	g_object_class_install_property (klass, OBJECT_PROP_POSITION_IS_MANUAL,
-		g_param_spec_boolean ("is-position-manual", "Is position manual", 
-				      "Is position manual",
-				      FALSE, G_PARAM_READWRITE|GOG_PARAM_PERSISTENT));
+		g_param_spec_boolean ("is-position-manual", 
+			_("Is position manual"), 
+			_("Is position manual"),
+			FALSE, 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	g_object_class_install_property (klass, OBJECT_PROP_POSITION_ANCHOR,
-		g_param_spec_string ("anchor", "Anchor",
-				     "Anchor for manual position",
-				     "top-left", G_PARAM_READWRITE|GOG_PARAM_PERSISTENT));
+		g_param_spec_string ("anchor", 
+			_("Anchor"),
+			_("Anchor for manual position"),
+			"top-left", 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT | GOG_PARAM_POSITION));
 
 	gog_object_signals [CHILD_ADDED] = g_signal_new ("child-added",
 		G_TYPE_FROM_CLASS (klass),
@@ -838,8 +928,10 @@ gog_object_set_id (GogObject *obj, unsigned id)
 
 	g_return_if_fail (IS_GOG_OBJECT (obj));
 
-	if (id == 0)
-		return gog_object_generate_id (obj);
+	if (id == 0) {
+		gog_object_generate_id (obj);
+		return;
+	}
 
 	g_return_if_fail (GOG_OBJECT (obj)->parent != NULL);
 
@@ -888,7 +980,7 @@ dataset_dup (GogDataset const *src, GogDataset *dst)
 GogObject *
 gog_object_dup (GogObject const *src, GogObject *new_parent, GogDataDuplicator datadup)
 {
-	gint	     n;
+	guint	     n;
 	GParamSpec **props;
 	GogObject   *dst = NULL;
 	GSList      *ptr;
@@ -1535,12 +1627,19 @@ gog_object_add_by_role (GogObject *parent, GogObjectRole const *role, GogObject
 
 	g_return_val_if_fail (is_a != 0, NULL);
 
-	if (child == NULL)
+	if (child == NULL) {
 		child = (role->allocate)
 			? (role->allocate) (parent)
 			: g_object_new (is_a, NULL);
 
+		/* g_object_new or the allocator have already generated an
+		 * error message, just exit */
+		if (child == NULL)
+			return NULL;
+	}
+
 	g_return_val_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (child, is_a), NULL);
+
 	child->explicitly_typed_role = explicitly_typed_role;
 	if (gog_object_set_parent (child, parent, role, 0))
 		return child;
@@ -1713,6 +1812,36 @@ gog_object_get_manual_allocation (GogObject *gobj,
 	return pos;
 }
 
+/*
+ * gog_object_is_default_position_flags:
+ * @obj: a #GogObject
+ * @name: name of the position property
+ *
+ * returns: true if the current position flags corresponding to the property @name
+ * are the defaults stored in @obj role.
+ **/
+gboolean
+gog_object_is_default_position_flags (GogObject const *obj, char const *name)
+{
+	int mask;
+
+	g_return_val_if_fail (name != NULL, FALSE);
+
+	if (obj->role == NULL)
+		return FALSE;
+
+	if (strcmp (name, "compass") == 0)
+		mask = GOG_POSITION_COMPASS;
+	else if (strcmp (name, "alignment") == 0)
+		mask = GOG_POSITION_ALIGNMENT;
+	else if (strcmp (name, "anchor") == 0)
+		mask = GOG_POSITION_ANCHOR;
+	else 
+		return FALSE;
+
+	return (obj->position & mask) == (obj->role->default_position & mask);
+}
+
 GogObjectRole const *
 gog_object_find_role_by_name (GogObject const *obj, char const *role)
 {
