@@ -872,8 +872,9 @@ cb_font_changed (GOFontSel *fs, PangoAttrList *list,
 	PangoAttrIterator *iter = pango_attr_list_get_iterator (list);
 	PangoFontDescription *desc = pango_font_description_new ();
 	GSList *extra_attrs;
-	const GOFont *font = go_font_new_by_desc (desc);
+	const GOFont *font;
 	pango_attr_iterator_get_font (iter, desc, NULL, &extra_attrs);
+	font = go_font_new_by_desc (desc);
 /* FIXME FIXME FIXME we should do something for extra attributes */
 	g_slist_foreach (extra_attrs, (GFunc)pango_attribute_destroy, NULL);
 	g_slist_free (extra_attrs);
@@ -1242,7 +1243,7 @@ static struct {
 };
 
 static struct {
-	GogFillStyle fstyle;
+	GogImageType fstyle;
 	char const *name;
 } image_tiling_names[] = {
 	{ GOG_IMAGE_CENTERED,     "centered" },
@@ -1308,7 +1309,7 @@ str_as_image_tiling (char const *name)
 }
 
 static char const *
-image_tiling_as_str (GogFillStyle fstyle)
+image_tiling_as_str (GogImageType fstyle)
 {
 	unsigned i;
 	for (i = 0; i < G_N_ELEMENTS (image_tiling_names); i++)
@@ -1358,7 +1359,7 @@ gog_style_line_sax_save (GsfXMLOut *output, char const *name,
 	gsf_xml_out_add_cstr_unchecked (output, "dash",
 		go_line_dash_as_str (line->dash_type));
 	gsf_xml_out_add_bool (output, "auto-dash", line->auto_dash);
-	gsf_xml_out_add_float (output, "width", line->width, 1);
+	gsf_xml_out_add_float (output, "width", line->width, 6);
 	go_xml_out_add_color (output, "color", line->color);
 	gsf_xml_out_add_bool (output, "auto-color", line->auto_color);
 	gsf_xml_out_end_element (output);
@@ -1397,7 +1398,7 @@ gog_style_gradient_sax_save (GsfXMLOut *output, GogStyle const *style)
 		style->fill.pattern.back);
 	if (style->fill.gradient.brightness >= 0.)
 		gsf_xml_out_add_float (output, "brightness",
-			style->fill.gradient.brightness, 2);
+			style->fill.gradient.brightness, 6);
 	else 
 		go_xml_out_add_color (output, "end-color",
 			style->fill.pattern.fore);
@@ -1660,7 +1661,7 @@ static void
 gog_style_text_layout_sax_save (GsfXMLOut *output, GogStyle const *style)
 {
 	gsf_xml_out_start_element (output, "text_layout");
-	gsf_xml_out_add_float (output, "angle", style->text_layout.angle, 1);
+	gsf_xml_out_add_float (output, "angle", style->text_layout.angle, 6);
 	gsf_xml_out_end_element (output);
 }
 
