@@ -136,7 +136,12 @@ go_math_init (void)
 	if (!(isnanl (go_nanl) &&
 	      go_pinfl > 0 && !finitel (go_pinfl) &&
 	      go_ninfl < 0 && !finitel (go_ninfl))) {
-		g_error ("Failed to generate long double NaN/+Inf/-Inf.  Please report at %s",
+		g_error ("Failed to generate long double NaN/+Inf/-Inf.\n"
+			 "    go_nanl=%.20Lg\n"
+			 "    go_pinfl=%.20Lg\n"
+			 "    go_ninfl=%.20Lg\n"
+			 "Please report at %s",
+			 go_nanl, go_pinfl, go_ninfl,
 			 bug_url);
 		abort ();
 	}
