@@ -39,7 +39,6 @@
 #include <goffice/app/module-plugin-defs.h>
 
 #include <glib/gi18n-lib.h>
-#include <gtk/gtklabel.h>
 #include <gsf/gsf-impl-utils.h>
 
 GOFFICE_PLUGIN_MODULE_HEADER;
@@ -47,7 +46,7 @@ GOFFICE_PLUGIN_MODULE_HEADER;
 enum {
 	GOG_1_5D_PROP_0,
 	GOG_1_5D_PROP_TYPE,
-	GOG_1_5D_PROP_IN_3D	/* place holder for XL */
+	GOG_1_5D_PROP_IN_3D	/* placeholder for XL */
 };
 
 static GogObjectClass *plot1_5d_parent_klass;
@@ -295,7 +294,6 @@ gog_plot1_5d_axis_get_bounds (GogPlot *plot, GogAxisType axis,
 		return NULL;
 	}
 
-	g_warning ("not reached");
 	return NULL;
 }
 
@@ -318,13 +316,17 @@ gog_plot1_5d_class_init (GogPlotClass *plot_klass)
 	gobject_klass->finalize = gog_plot1_5d_finalize;
 
 	g_object_class_install_property (gobject_klass, GOG_1_5D_PROP_TYPE,
-		g_param_spec_string ("type", "type",
-			"How to group multiple series, normal, stacked, as_percentage",
-			"normal", G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+		g_param_spec_string ("type", 
+			_("Type"),
+			_("How to group multiple series, normal, stacked, as_percentage"),
+			"normal", 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, GOG_1_5D_PROP_IN_3D,
-		g_param_spec_boolean ("in-3d", "in-3d",
-			"Place holder to all us to round trip pseudo 3d state",
-			FALSE, G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+		g_param_spec_boolean ("in-3d", 
+			_("In 3d"),
+			_("Placeholder to allow us to round trip pseudo 3d state"),
+			FALSE, 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 
 	gog_klass->update	= gog_plot1_5d_update;
 
@@ -435,6 +437,8 @@ lines_post_add (GogObject *parent, GogObject *child)
 {
 	GogSeries1_5d *series = GOG_SERIES1_5D (parent);
 	series->has_lines = TRUE;
+	if (GOG_IS_PLOT_DROPBAR (series->base.plot))
+		gog_series_lines_use_markers (GOG_SERIES_LINES (child), TRUE);
 	gog_object_request_update (child);
 }
 
@@ -538,6 +542,8 @@ gog_series1_5d_get_property (GObject *obj, guint param_id,
 	}
 }
 
+#ifdef GOFFICE_WITH_GTK
+#include <gtk/gtklabel.h>
 static void 
 gog_series1_5d_populate_editor (GogObject *obj,
 				GogEditor *editor,
@@ -560,6 +566,7 @@ gog_series1_5d_populate_editor (GogObject *obj,
 	error_page = gog_error_bar_prefs (series, "errors", horizontal, dalloc, cc);
 	gog_editor_add_page (editor, error_page, _("Error bars"));
 }
+#endif
 
 static void
 gog_series1_5d_finalize (GObject *obj)
@@ -610,15 +617,19 @@ gog_series1_5d_class_init (GogObjectClass *obj_klass)
 	gobject_klass->get_property = gog_series1_5d_get_property;
 	gobject_klass->finalize 	      = gog_series1_5d_finalize;
 	obj_klass->update 	      = gog_series1_5d_update;
+#ifdef GOFFICE_WITH_GTK
 	obj_klass->populate_editor    = gog_series1_5d_populate_editor;
+#endif
 	gog_series_klass->dim_changed = gog_series1_5d_dim_changed;
 
 	gog_object_register_roles (obj_klass, roles, G_N_ELEMENTS (roles));
 
 	g_object_class_install_property (gobject_klass, SERIES_PROP_ERRORS,
-		g_param_spec_object ("errors", "errors",
-			"GogErrorBar *",
-			GOG_ERROR_BAR_TYPE, G_PARAM_READWRITE|GOG_PARAM_PERSISTENT));
+		g_param_spec_object ("errors", 
+			_("Error bars"),
+			_("GogErrorBar *"),
+			GOG_ERROR_BAR_TYPE, 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 }
 
 static void
@@ -650,6 +661,7 @@ go_plugin_init (GOPlugin *plugin, GOCmdContext *cc)
 	gog_dropbar_plot_register_type (module);
 	gog_dropbar_view_register_type (module);
 	gog_line_series_register_type (module);
+	gog_line_series_view_register_type (module);
 	gog_line_plot_register_type (module);
 	gog_area_plot_register_type (module);
 	gog_line_view_register_type (module);
