@@ -325,21 +325,29 @@ gog_xy_plot_class_init (GogPlotClass *plot_klass)
 	gobject_klass->get_property = gog_xy_get_property;
 
 	g_object_class_install_property (gobject_klass, GOG_XY_PROP_DEFAULT_STYLE_HAS_MARKERS,
-		g_param_spec_boolean ("default-style-has-markers", NULL,
-			"Should the default style of a series include markers",
-			TRUE, G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+		g_param_spec_boolean ("default-style-has-markers", 
+			_("Default markers"),
+			_("Should the default style of a series include markers"),
+			TRUE, 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, GOG_XY_PROP_DEFAULT_STYLE_HAS_LINES,
-		g_param_spec_boolean ("default-style-has-lines", NULL,
-			"Should the default style of a series include lines",
-			TRUE, G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+		g_param_spec_boolean ("default-style-has-lines", 
+			_("Default lines"),
+			_("Should the default style of a series include lines"),
+			TRUE, 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, GOG_XY_PROP_USE_SPLINES,
-		g_param_spec_boolean ("use-splines", NULL,
-			"Should the plot use splines instead of linear interpolation",
-			FALSE, G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+		g_param_spec_boolean ("use-splines", 
+			_("Use splines"),
+			_("Should the plot use splines instead of linear interpolation"),
+			FALSE, 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, GOG_XY_PROP_INTERPOLATION,
-		g_param_spec_string  ("interpolation", NULL,
-			_("Interpolation type (none, linear, spline or step) with variant, if any."),
-            "none", G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+		g_param_spec_string  ("interpolation", 
+			_("Interpolation"),
+			_("Interpolation type (none, linear, spline or step) with variant, if any"),
+			"none", 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	gog_klass->type_name	= gog_xy_plot_type_name;
 
 	{
@@ -482,25 +490,29 @@ gog_bubble_plot_class_init (GogPlotClass *plot_klass)
 	gog_2d_plot_klass->adjust_bounds = gog_bubble_plot_adjust_bounds;
 
 	g_object_class_install_property (gobject_klass, GOG_BUBBLE_PROP_AS_AREA,
-		g_param_spec_boolean ("size-as-area", "size-as-area",
-			"Display size as area instead of diameter",
+		g_param_spec_boolean ("size-as-area", 
+			_("Size as area"),
+			_("Display size as area instead of diameter"),
 			TRUE,
-			G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, GOG_BUBBLE_PROP_SHOW_NEGATIVES,
-		g_param_spec_boolean ("show-negatives", "show-negatives",
-			"Draw bubbles for negative values",
+		g_param_spec_boolean ("show-negatives", 
+			_("Show negatives"),
+			_("Draw bubbles for negative values"),
 			FALSE,
-			G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, GOG_BUBBLE_PROP_IN_3D,
-		g_param_spec_boolean ("in-3d", "in-3d",
-			"Draw 3d bubbles",
+		g_param_spec_boolean ("in-3d", 
+			_("In 3d"),
+			_("Draw 3d bubbles"),
 			FALSE,
-			G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, GOG_BUBBLE_PROP_SCALE,
-		g_param_spec_float ("bubble-scale", "bubble-scale",
-			"Fraction of default radius used for display.",
+		g_param_spec_float ("bubble-scale", 
+			_("Bubble scale"),
+			_("Fraction of default radius used for display"),
 			0., 2., 1.,
-			G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	{
 		static GogSeriesDimDesc dimensions[] = {
 			{ N_("X"), GOG_SERIES_SUGGESTED, FALSE,
@@ -559,6 +571,7 @@ enum {
 	GOG_XY_COLOR_PROP_0,
 	GOG_XY_COLOR_PROP_DEFAULT_STYLE_HAS_LINES,
 	GOG_XY_COLOR_PROP_INTERPOLATION,
+	GOG_XY_COLOR_PROP_HIDE_OUTLIERS,
 };
 
 static GogObjectClass *map_parent_klass;
@@ -597,7 +610,6 @@ gog_xy_color_plot_update (GogObject *obj)
 		if (model->z.fmt == NULL)
 			model->z.fmt = go_data_preferred_fmt (series->base.values[2].data);
 	}
-
 	if (model->z.minima != z_min || model->z.maxima != z_max) {
 		model->z.minima = z_min;
 		model->z.maxima = z_max;
@@ -610,7 +622,7 @@ static GOData *
 gog_xy_color_plot_axis_get_bounds (GogPlot *plot, GogAxisType axis,
 			     GogPlotBoundInfo *bounds)
 {
-	if (axis == GOG_AXIS_PSEUDO_3D) {
+	if (axis == GOG_AXIS_COLOR) {
 		GogXYColorPlot *model = GOG_XY_COLOR_PLOT (plot);
 
 		bounds->val.minima = model->z.minima;
@@ -634,25 +646,70 @@ gog_xy_color_plot_type_name (G_GNUC_UNUSED GogObject const *item)
 	return N_("XYColor");
 }
 
+#ifdef GOFFICE_WITH_GTK
+#include <goffice/gtk/goffice-gtk.h>
+#include <gtk/gtktogglebutton.h>
+static void
+hide_outliers_toggled_cb (GtkToggleButton *btn, GObject *obj)
+{
+	g_object_set (obj, "hide-outliers", gtk_toggle_button_get_active (btn), NULL);
+}
+
+static void 
+gog_xy_color_plot_populate_editor (GogObject *obj,
+			       GogEditor *editor,
+			       GogDataAllocator *dalloc,
+			       GOCmdContext *cc)
+{
+	GtkWidget *w;
+	char const *dir = go_plugin_get_dir_name (
+		go_plugins_get_plugin_by_id ("GOffice_plot_xy"));
+	char *path = g_build_filename (dir, "gog-xy-color-prefs.glade", NULL);
+	GladeXML *gui = go_libglade_new (path, "gog-xy-color-prefs", GETTEXT_PACKAGE, cc);
+
+	g_free (path);
+
+	if (gui != NULL) {
+		w = glade_xml_get_widget (gui, "hide-outliers");
+		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (w),
+				(GOG_XY_COLOR_PLOT (obj))->hide_outliers);
+		g_signal_connect (G_OBJECT (w),
+			"toggled",
+			G_CALLBACK (hide_outliers_toggled_cb), obj);
+		w = glade_xml_get_widget (gui, "gog-xy-color-prefs");
+		gog_editor_add_page (editor, w, _("Properties"));
+	}
+
+	(GOG_OBJECT_CLASS(map_parent_klass)->populate_editor) (obj, editor, dalloc, cc);
+}
+#endif
+
 static void
 gog_xy_color_plot_set_property (GObject *obj, guint param_id,
 		     GValue const *value, GParamSpec *pspec)
 {
 	GogXYColorPlot *map = GOG_XY_COLOR_PLOT (obj);
 	switch (param_id) {
-	case GOG_XY_COLOR_PROP_DEFAULT_STYLE_HAS_LINES: {
+	case GOG_XY_COLOR_PROP_DEFAULT_STYLE_HAS_LINES:
 		map->default_style_has_lines = g_value_get_boolean (value);
 		break;
-	}
 	case GOG_XY_COLOR_PROP_INTERPOLATION: {
 		char const *s = g_value_get_string (value);
 		map->interpolation = go_line_interpolation_from_str (s);;
 		break;
 	}
+	case GOG_XY_COLOR_PROP_HIDE_OUTLIERS:
+		map->hide_outliers = g_value_get_boolean (value);
+		break;
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 break;
 	}
+
+	/* none of the attributes triggers a size change yet.
+	 * When we add data labels we'll need it */
+	gog_object_emit_changed (GOG_OBJECT (obj), FALSE);
 }
+
 static void
 gog_xy_color_plot_get_property (GObject *obj, guint param_id,
 		     GValue *value, GParamSpec *pspec)
@@ -665,6 +722,9 @@ gog_xy_color_plot_get_property (GObject *obj, guint param_id,
 	case GOG_XY_COLOR_PROP_INTERPOLATION:
 		g_value_set_string (value, go_line_interpolation_as_str (map->interpolation));
 		break;
+	case GOG_XY_COLOR_PROP_HIDE_OUTLIERS:
+		g_value_set_boolean (value, map->hide_outliers);
+		break;
 	default: G_OBJECT_WARN_INVALID_PROPERTY_ID (obj, param_id, pspec);
 		 break;
 	}
@@ -690,16 +750,26 @@ gog_xy_color_plot_class_init (GogPlotClass *plot_klass)
 	gobject_klass->finalize     = gog_xy_color_plot_finalize;
 
 	g_object_class_install_property (gobject_klass, GOG_XY_COLOR_PROP_DEFAULT_STYLE_HAS_LINES,
-		g_param_spec_boolean ("default-style-has-lines", NULL,
-			"Should the default style of a series include lines",
-			TRUE, G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+		g_param_spec_boolean ("default-style-has-lines", 
+			_("Default lines"),
+			_("Should the default style of a series include lines"),
+			TRUE, 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, GOG_XY_COLOR_PROP_INTERPOLATION,
-		g_param_spec_string  ("interpolation", NULL,
-			_("Interpolation type (none, linear, spline or step) with variant, if any."),
-            "none", G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
-	gog_klass->type_name	= gog_xy_color_plot_type_name;
+		g_param_spec_string  ("interpolation", 
+			_("Interpolation"),
+			_("Interpolation type (none, linear, spline or step) with variant, if any"),
+			"none", 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+	g_object_class_install_property (gobject_klass, GOG_XY_COLOR_PROP_HIDE_OUTLIERS,
+		g_param_spec_boolean  ("hide-outliers", 
+			_("hide-outliers"),
+			_("Hide data outside of the color axis bounds"),
+			TRUE, 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
+	gog_klass->type_name		= gog_xy_color_plot_type_name;
 	gog_klass->update		= gog_xy_color_plot_update;
-
+	gog_klass->populate_editor	= gog_xy_color_plot_populate_editor;
 	{
 		static GogSeriesDimDesc dimensions[] = {
 			{ N_("X"), GOG_SERIES_SUGGESTED, FALSE,
@@ -731,6 +801,7 @@ static void
 gog_xy_color_plot_init (GogXYColorPlot *map)
 {
 	map->default_style_has_lines = FALSE;
+	map->hide_outliers = TRUE;
 }
 
 GSF_DYNAMIC_CLASS (GogXYColorPlot, gog_xy_color_plot,
@@ -766,8 +837,10 @@ bubble_draw_circle (GogView *view, double x, double y, double radius)
 }
 
 static GOColor
-get_map_color (double z)
+get_map_color (double z, gboolean hide_outliers)
 {
+	if (hide_outliers && (z < 0. || z > 6.))
+		return 0;
 	if (z <= 0.)
 		return RGBA_BLUE;
 	if (z <= 1.)
@@ -807,7 +880,7 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 	double xerrmin, xerrmax, yerrmin, yerrmax;
 	GogStyle *style = NULL;
 	gboolean show_marks, show_lines, show_negatives, in_3d, size_as_area = TRUE;
-	gboolean is_map = GOG_IS_XY_COLOR_PLOT (model);
+	gboolean is_map = GOG_IS_XY_COLOR_PLOT (model), hide_outliers = TRUE;
 
 	MarkerData **markers;
 	unsigned *num_markers;
@@ -827,6 +900,8 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 	z_map = is_map?
 			gog_axis_map_new (model->base.axis[GOG_AXIS_COLOR], 0, 6):
 			NULL;
+	if (is_map)
+		hide_outliers = GOG_XY_COLOR_PLOT (model)->hide_outliers;
 
 	/* Draw drop lines from point to axis start. To change this behaviour
 	 * and draw drop lines from point to zero, we can use gog_axis_map_get_baseline:
@@ -1140,7 +1215,7 @@ gog_xy_view_render (GogView *view, GogViewAllocation const *bbox)
 				markers[j][k].y = y_canvas;
 				if (is_map)
 					markers[j][k].color = (gog_axis_map_finite (z_map, z))?
-							get_map_color (gog_axis_map_to_view (z_map, z)):
+							get_map_color (gog_axis_map_to_view (z_map, z), hide_outliers):
 							0;
 				k++;
 			}
@@ -1501,8 +1576,6 @@ gog_xy_series_get_property (GObject *obj, guint param_id,
 }
 
 #ifdef GOFFICE_WITH_GTK
-#include <goffice/gtk/goffice-gtk.h>
-#include <gtk/gtktogglebutton.h>
 static void
 invalid_toggled_cb (GtkToggleButton *btn, GObject *obj)
 {
@@ -1585,17 +1658,23 @@ gog_xy_series_class_init (GogStyledObjectClass *gso_klass)
 	gog_object_register_roles (gog_klass, roles, G_N_ELEMENTS (roles));
 
 	g_object_class_install_property (gobject_klass, SERIES_PROP_XERRORS,
-		g_param_spec_object ("x-errors", "x-errors",
-			"GogErrorBar *",
-			GOG_ERROR_BAR_TYPE, G_PARAM_READWRITE|GOG_PARAM_PERSISTENT));
+		g_param_spec_object ("x-errors", 
+			_("X error bars"),
+			_("GogErrorBar *"),
+			GOG_ERROR_BAR_TYPE, 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, SERIES_PROP_YERRORS,
-		g_param_spec_object ("y-errors", "y-errors",
-			"GogErrorBar *",
-			GOG_ERROR_BAR_TYPE, G_PARAM_READWRITE|GOG_PARAM_PERSISTENT));
+		g_param_spec_object ("y-errors", 
+			_("Y error bars"),
+			_("GogErrorBar *"),
+			GOG_ERROR_BAR_TYPE, 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE | GOG_PARAM_PERSISTENT));
 	g_object_class_install_property (gobject_klass, SERIES_PROP_INVALID_AS_ZERO,
-		g_param_spec_boolean ("invalid-as-zero", _("invalid-as-zero"),
+		g_param_spec_boolean ("invalid-as-zero", 
+			_("Invalid as zero"),
 			_("Replace invalid values by 0 when drawing markers or bubbles"),
-			FALSE, G_PARAM_READWRITE|GOG_PARAM_PERSISTENT));
+			FALSE, 
+			GSF_PARAM_STATIC | G_PARAM_READWRITE|GOG_PARAM_PERSISTENT));
 }
 
 GSF_DYNAMIC_CLASS (GogXYSeries, gog_xy_series,
